/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2014  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;

const GLib = imports.gi.GLib;
const Shell = imports.gi.Shell;
const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;
const PanelTools = Extension.imports.panelTools;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    PanelManager - BasicSettings
 */
const StatusArea = Main.panel.statusArea || Main.panel._statusArea;

const DEFAULT_BASIC_SETTINGS = [
    [ _("System Settings"), 'gnome-control-center', 'preferences-system' ],
    [ _("GNOME Shell Extensions"), 'gnome-shell-extension-prefs', 'preferences-other' ],
    [ _("Startup Applications"), 'gnome-session-properties', 'session-properties' ],
    [ _("GNOME Tweak Tool"), 'gnome-tweak-tool', 'preferences-system' ],
    [ _("Synaptic Package Manager"), 'synaptic-pkexec', 'synaptic' ],
    [ _("Configure printers"), 'system-config-printer', 'printer' ],
    [ _("Users Settings"), 'users-admin', 'config-users' ]
];

const DEFAULT_EXTENSION = [
    [ _("Settings for PanelManager"), Extension['uuid'], 'preferences-other' ],
    [ _("Settings for Temperature"), 'temperature@extensions.dschaerf.de', 'preferences-other' ],
    [ _("Settings for Weather"), 'weather@extensions.dschaerf.de', 'preferences-other' ]
];

const KEY_SHOW_STATUSAREA = 'show-statusarea';
const KEY_SHOW_STATUSMENU = 'show-statusmenu';

const STATUSAREA_ADVANCED = 1;
const STATUSAREA_ALTERNATE = 2;

const STATUSMENU_DEFAULT = 0;

const PopupBasicSettingsMenuSection = new Lang.Class({
    Name: 'PopupBasicSettingsMenuSection',
    Extends: PopupMenu.PopupMenuSection,

    _init: function(wantIcon) {
        this.parent();
        this._wantIcon = wantIcon || false;
        this._appSystem = Shell.AppSystem.get_default();
        this._connectAppSystem = this._appSystem.connect('installed-changed', Lang.bind(this, this._onInstalledChanged));
        this._onInstalledChanged();
        this.actor.connect('destroy', Lang.bind(this, function() {
            this._appSystem.disconnect(this._connectAppSystem);
        }));
    },

    _onInstalledChanged: function() {
        this.removeAll();
        let apps = [];
        for each(let app in DEFAULT_BASIC_SETTINGS) {
            if (GLib.find_program_in_path(app[1])) apps.push(app);
        }
        apps.sort(function(a, b) {
            return PanelTools.getSorted(a[0], b[0]);
        });
        for (let i = 0; i < apps.length; i++) {
            let app = apps[i];
            let item = new PanelTools.PopupMenuItem(app[0], this._wantIcon);
            if (this._wantIcon) item.setIconName(app[2]);
            item.connect('activate', Lang.bind(this, function() {
                Main.overview.hide();
                Main.Util.spawnCommandLine(app[1]);
            }));
            this.addMenuItem(item);
        }
    }
});

const PopupExtensionsMenuSection = new Lang.Class({
    Name: 'PopupExtensionsMenuSection',
    Extends: PopupMenu.PopupMenuSection,

    _init: function(wantIcon) {
        this.parent();
        this._wantIcon = wantIcon || false;
        this._appSystem = Shell.AppSystem.get_default();
        this._connectAppSystem = this._appSystem.connect('installed-changed', Lang.bind(this, this._onInstalledChanged));
        this._onInstalledChanged();
        this.actor.connect('destroy', Lang.bind(this, function() {
            this._appSystem.disconnect(this._connectAppSystem);
        }));
    },

    _onInstalledChanged: function() {
        this.removeAll();
        let apps = [];
        for each(let app in DEFAULT_EXTENSION) {
            if (PanelTools.checkExtension(app[1])) apps.push(app);
        }
        apps.sort(function(a, b) {
            return PanelTools.getSorted(a[0], b[0]);
        });
        for (let i = 0; i < apps.length; i++) {
            let app = apps[i];
            let item = new PanelTools.PopupMenuItem(app[0], this._wantIcon);
            if (this._wantIcon) item.setIconName(app[2]);
            item.connect('activate', Lang.bind(this, function() {
                Main.overview.hide();
                Main.Util.spawnCommandLine('gnome-shell-extension-prefs '+app[1]);
            }));
            this.addMenuItem(item);
        }
    }
});

const BasicSettingsSystemIndicator = new Lang.Class({
    Name: 'BasicSettingsSystemIndicator',
    Extends: PanelMenu.SystemIndicator || PanelTools.SystemIndicator,

    _init: function() {
        this.parent();
        this._indicator = this._addIndicator();
        this._indicator.icon_name = 'preferences-system-symbolic';
        this._item = new PanelTools.PopupSubMenuMenuItem(_("BasicSettings"), true, { settings: false });
        this._item.menu.addMenuItem(new PopupBasicSettingsMenuSection());
        this._item.setIconName('preferences-system-symbolic');
        this.menu.addMenuItem(this._item);
    },

    destroy: function() {
        this.menu.destroy();
        this.indicators.destroy();
    }
});

const BasicSettingsIndicator = new Lang.Class({
    Name: 'BasicSettingsIndicator',
    Extends: PanelTools.Button,

    _init: function() {
        this.parent(0.0, _("BasicSettings"), { icon_name: 'preferences-system-symbolic' });
        this.menu.addMenuItem(new PopupBasicSettingsMenuSection(true));
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this.menu.addMenuItem(new PopupExtensionsMenuSection());
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_STATUSAREA, Lang.bind(this, this._onShowStatusArea));
        this._onShowStatusArea();
        this.actor.connect('destroy', Lang.bind(this, function() {
            this._settings.disconnect(this._connectSettings);
        }));
    },

    _onShowStatusArea: function() {
        this.actor.visible = this._settings.get_enum(KEY_SHOW_STATUSAREA) === STATUSAREA_ADVANCED;
    }
});

function getExtension() {
    return PanelTools.addToStatusArea('dschaerfBasicSettings', new BasicSettingsIndicator(), PanelTools.getChildren('right') - 1, 'right');
}
