/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2014  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;
const Mainloop = imports.mainloop;

const Clutter = imports.gi.Clutter;
const GLib = imports.gi.GLib;
const Shell = imports.gi.Shell;
const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;
const PanelTools = Extension.imports.panelTools;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    Temperature
 */
const KEY_DISPLAY_TEMPERATURE = 'display-temperature';
const KEY_REFRESH_INTERVAL = 'refresh-interval';
const KEY_SHOW_CPU_CORE_TEMPERATURE = 'show-cpu-core-temperature';
const KEY_SHOW_DISK_TEMPERATURE = 'show-disk-temperature';
const KEY_SHOW_FAHRENHEIT = 'show-fahrenheit';
const KEY_SHOW_GRAPHICS_TEMPERATURE = 'show-graphics-temperature';
const KEY_SHOW_SETTINGS = 'show-settings';

const TEMPERATURE_INDICATOR_CPU = 0;
const TEMPERATURE_INDICATOR_CPU_CORES = 1;
const TEMPERATURE_INDICATOR_VGA = 2;
const TEMPERATURE_INDICATOR_CPU_VGA = 3;

const TEMPERATURE_MENUITEM_NONE = 0;
const TEMPERATURE_MENUITEM_CURRENT = 1;
const TEMPERATURE_MENUITEM_CRITICAL = 2;
const TEMPERATURE_MENUITEM_MAXIMUM = 3;
const TEMPERATURE_MENUITEM_ALL = 4;

const TRANSLATIONS = [
    _("CPU Temperature"),
    _("MB Temperature")
];

const Indicator = new Lang.Class({
    Name: 'TemperatureIndicator',
    Extends: PanelTools.Button,

    _init: function() {
        this.parent(0.0, _("Temperature"), { text: '--' });
        this.event = null;
        this._appSystem = Shell.AppSystem.get_default();
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._createSubMenu();
        this._checkSensors();
        this._onShowSettings();
        this._connectAppSystem = this._appSystem.connect('installed-changed', Lang.bind(this, this._checkSensors));
        this._connectSettings = [
            this._settings.connect('changed::'+KEY_DISPLAY_TEMPERATURE, Lang.bind(this, this._onDisplayTemperature)),
            this._settings.connect('changed::'+KEY_REFRESH_INTERVAL, Lang.bind(this, this._onRefreshInterval)),
            this._settings.connect('changed::'+KEY_SHOW_CPU_CORE_TEMPERATURE, Lang.bind(this, this._onShowCPUCoreTemperature)),
            this._settings.connect('changed::'+KEY_SHOW_DISK_TEMPERATURE, Lang.bind(this, this._onShowDiskTemperature)),
            this._settings.connect('changed::'+KEY_SHOW_FAHRENHEIT, Lang.bind(this, this._onShowFahrenheit)),
            this._settings.connect('changed::'+KEY_SHOW_GRAPHICS_TEMPERATURE, Lang.bind(this, this._onShowGraphicsTemperature)),
            this._settings.connect('changed::'+KEY_SHOW_SETTINGS, Lang.bind(this, this._onShowSettings))
        ];
        this.actor.connect('destroy', Lang.bind(this, function() {
            if (this.event) Mainloop.source_remove(this.event);
            for each(let connection in this._connectSettings) {
                this._settings.disconnect(connection);
            }
            this._appSystem.disconnect(this._connectAppSystem);
        }));
    },

    _checkATI: function() {
        this._commandATI = GLib.find_program_in_path('aticonfig');
    },

    _checkHDDTemp: function() {
        this._commandHDDTemp =  GLib.find_program_in_path('hddtemp');
        if (this._commandHDDTemp && GLib.spawn_command_line_sync(this._commandHDDTemp)[3]) this._commandHDDTemp = null;
        this._daemonHDDTemp = null;
        let ret = GLib.spawn_command_line_sync('pidof hddtemp');
        if (ret[1].length) {
            let cmdline = GLib.spawn_command_line_sync('ps --pid='+ret[1]+' -o args=');
            if (cmdline.indexOf(' -p ')) this._daemonHDDTemp = cmdline.toString().split(' -p ')[1].split(' ')[0];
        }
    },

    _checkHWMon: function() {
        this._dirHWMon = [];
        for (let i = 0; ; i++) {
            let dirHWMon = '/sys/class/hwmon/hwmon'+i+'/';
            let dirHWMonDevice = dirHWMon+'device/';
            if (GLib.file_test(dirHWMon+'name', GLib.FileTest.EXISTS)) {
                this._dirHWMon.push(dirHWMon);
            } else if (GLib.file_test(dirHWMonDevice+'name', GLib.FileTest.EXISTS)) {
                this._dirHWMon.push(dirHWMonDevice);
            } else {
                if (i > 0) break;
            }
        }
    },

    _checkNVIDIA: function() {
        this._commandNVIDIA = GLib.find_program_in_path('nvidia-settings');
    },

    _checkSensors: function() {
        this._checkATI();
        this._checkHDDTemp();
        this._checkHWMon();
        this._checkNVIDIA();
        this._onRefreshInterval();
    },

    _createSubMenu: function() {
        this._itemNotAvailable = new PopupMenu.PopupSwitchMenuItem(_("No data available"));
        this._itemNotAvailable.setStatus('--');
        this.menu.addMenuItem(this._itemNotAvailable);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemHWMonACPITZ = new PopupMenu.PopupMenuSection();
        this.menu.addMenuItem(this._itemHWMonACPITZ);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemCPUCores = new PopupMenu.PopupMenuSection();
        this.menu.addMenuItem(this._itemCPUCores);
        this._itemHWMonDevice = new PopupMenu.PopupMenuSection();
        this.menu.addMenuItem(this._itemHWMonDevice);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemHDDTemp = new PopupMenu.PopupMenuSection();
        this.menu.addMenuItem(this._itemHDDTemp);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemGraphicCards = new PopupMenu.PopupMenuSection();
        this.menu.addMenuItem(this._itemGraphicCards);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemSettings = this.menu.addAction(_("Settings for Temperature"), Lang.bind(this, function() {
            Main.overview.hide();
            Main.Util.spawnCommandLine('gnome-shell-extension-prefs '+Extension['uuid']);
        }));
    },

    _formatTemp: function(temp) {
        if (!temp) return '--';
        if (this._settings.get_boolean(KEY_SHOW_FAHRENHEIT)) {
            return (Math.round((temp*(9/5)+32).toFixed(1))).toString()+'\u00b0F';
        } else {
            return (Math.round(temp)).toString()+'\u00b0C';
        }
    },

    _getContent: function(file) {
        try {
            return GLib.file_get_contents(file);
        } catch (e) {
            return [];
        }
    },

    _getHDDTemp: function(text, isDaemon) {
        let hddtempOutput = text.split('\n');
        let s = [];
        let n = 0;
        for (let i = 0; i < hddtempOutput.length; i++) {
            if (hddtempOutput[i]) {
                s[++n] = [];
                if (isDaemon) {
                    s[n]['label'] = hddtempOutput[i].split('|')[1].split('/');
                } else {
                    s[n]['label'] = hddtempOutput[i].split(': ')[0].split('/');
                }
                s[n]['label'] = s[n]['label'][s[n]['label'].length-1];
                if (isDaemon) {
                    s[n]['temp'] = parseFloat(hddtempOutput[i].split('|')[3]);
                } else {
                    s[n]['temp'] = parseFloat(hddtempOutput[i].split(': ')[2]);
                }
            }
        }
        return s;
    },

    _getLabel: function() {
        switch (this._settings.get_enum(KEY_DISPLAY_TEMPERATURE)) {
            case TEMPERATURE_INDICATOR_CPU:
                return this._formatTemp(this._tempCPU);
            case TEMPERATURE_INDICATOR_CPU_CORES:
                return this._formatTemp(this._tempCPUCores);
            case TEMPERATURE_INDICATOR_VGA:
                return this._formatTemp(this._tempGraphics);
            case TEMPERATURE_INDICATOR_CPU_VGA:
                return this._formatTemp(this._tempCPU)+' / '+this._formatTemp(this._tempGraphics);
            default:
                return '--';
        }
    },

    _onDisplayTemperature: function() {
        this.label.text = this._getLabel();
    },

    _onRefreshInterval: function() {
        if (this.event) Mainloop.source_remove(this.event);
        this.event = Mainloop.timeout_add_seconds(this._settings.get_string(KEY_REFRESH_INTERVAL), Lang.bind(this, function() {
            this._updateSensors();
            return true;
        }));
        this._updateSensors();
    },

    _onShowCPUCoreTemperature: function() {
        let showCPUCore = this._settings.get_enum(KEY_SHOW_CPU_CORE_TEMPERATURE);
        this._itemCPUCores.actor.visible = showCPUCore !== TEMPERATURE_MENUITEM_NONE && this._itemCPUCores.numMenuItems > 0;
        if (this._itemCPUCores.actor.visible) {
            let menuItems = this._itemCPUCores._getMenuItems();
            for (let i = 0; i < menuItems.length; i++) {
                let status = this._formatTemp(this._infoCPUCores[i]['current']);
                if (showCPUCore !== TEMPERATURE_MENUITEM_CURRENT) {
                    if (this._infoCPUCores[i]['maximum'] && (showCPUCore === TEMPERATURE_MENUITEM_MAXIMUM || showCPUCore === TEMPERATURE_MENUITEM_ALL)) status += ' / '+this._formatTemp(this._infoCPUCores[i]['maximum']);
                    if (this._infoCPUCores[i]['critical'] && (showCPUCore === TEMPERATURE_MENUITEM_CRITICAL || showCPUCore === TEMPERATURE_MENUITEM_ALL)) status += ' / '+this._formatTemp(this._infoCPUCores[i]['critical']);
                }
                menuItems[i].setStatus(status);
            }
        }
    },

    _onShowDiskTemperature: function() {
        let showDisk = this._settings.get_boolean(KEY_SHOW_DISK_TEMPERATURE);
        this._itemHDDTemp.actor.visible = showDisk && this._itemHDDTemp.numMenuItems > 0;
        if (this._itemHDDTemp.actor.visible) {
            let menuItems = this._itemHDDTemp._getMenuItems();
            for (let i = 0; i < menuItems.length; i++) {
                let status = this._formatTemp(this._infoHDDTemp[i]);
                menuItems[i].setStatus(status);
            }
        }
    },

    _onShowHWMonDevice: function() {
        let showHWMonDevice = this._settings.get_enum(KEY_SHOW_CPU_CORE_TEMPERATURE);
        this._itemHWMonDevice.actor.visible = showHWMonDevice !== TEMPERATURE_MENUITEM_NONE && this._itemHWMonDevice.numMenuItems > 0;
        if (this._itemHWMonDevice.actor.visible) {
            let menuItems = this._itemHWMonDevice._getMenuItems();
            for (let i = 0; i < menuItems.length; i++) {
                let status = this._formatTemp(this._infoHWMonDevice[i]['current']);
                if (showHWMonDevice !== TEMPERATURE_MENUITEM_CURRENT) {
                    if (this._infoHWMonDevice[i]['maximum'] && (showHWMonDevice === TEMPERATURE_MENUITEM_MAXIMUM || showHWMonDevice === TEMPERATURE_MENUITEM_ALL)) status += ' / '+this._formatTemp(this._infoHWMonDevice[i]['maximum']);
                    if (this._infoHWMonDevice[i]['critical'] && (showHWMonDevice === TEMPERATURE_MENUITEM_CRITICAL || showHWMonDevice === TEMPERATURE_MENUITEM_ALL)) status += ' / '+this._formatTemp(this._infoHWMonDevice[i]['critical']);
                }
                menuItems[i].setStatus(status);
            }
        }
    },

    _onShowFahrenheit: function() {
        this._onDisplayTemperature();
        let menuItems = this._itemHWMonACPITZ._getMenuItems();
        for (let i = 0; i < menuItems.length; i++) {
            menuItems[i].setStatus(this._formatTemp(this._infoHWMonACPITZ[i]));
        }
        this._itemHWMonACPITZ.actor.visible = this._itemHWMonACPITZ.numMenuItems > 0;
        this._onShowCPUCoreTemperature()
        this._onShowHWMonDevice();
        this._onShowDiskTemperature();
        this._onShowGraphicsTemperature();
    },

    _onShowGraphicsTemperature: function() {
        let showGraphics = this._settings.get_enum(KEY_SHOW_GRAPHICS_TEMPERATURE);
        this._itemGraphicCards.actor.visible = showGraphics !== TEMPERATURE_MENUITEM_NONE && this._itemGraphicCards.numMenuItems > 0;
        if (this._itemGraphicCards.actor.visible) {
            let menuItems = this._itemGraphicCards._getMenuItems();
            for (let i = 0; i < menuItems.length; i++) {
                let status = this._formatTemp(this._infoGraphics[i]['current']);
                if (showGraphics !== TEMPERATURE_MENUITEM_CURRENT) {
                    if (this._infoGraphics[i]['maximum'] && (showGraphics === TEMPERATURE_MENUITEM_MAXIMUM || showGraphics === TEMPERATURE_MENUITEM_ALL)) status += ' / '+this._formatTemp(this._infoGraphics[i]['maximum']);
                    if (this._infoGraphics[i]['critical'] && (showGraphics === TEMPERATURE_MENUITEM_CRITICAL || showGraphics === TEMPERATURE_MENUITEM_ALL)) status += ' / '+this._formatTemp(this._infoGraphics[i]['critical']);
                }
                menuItems[i].setStatus(status);
            }
        }
    },

    _onShowSettings: function() {
        this._itemSettings.actor.visible = this._settings.get_boolean(KEY_SHOW_SETTINGS) || this._itemNotAvailable.actor.visible;
    },

    _updateATI: function(showGraphicsTemperature) {
        let atiOutput = GLib.spawn_command_line_sync(this._commandATI+" --od-gettemperature | sed -n '/Temperatur/{s/.*- //; s/[.].*//; p}'")
        if (atiOutput[0]) {
            let atiTemp = atiOutput[1].toString().split('\n')[0];
            if (atiTemp > 0) {
                if (!this._tempGraphics || this._tempGraphics < atiTemp) this._tempGraphics = atiTemp;
                this._infoGraphics.push({ current: nvidiaTemp, maximum: null, critical: null });
                this._itemGraphicCards.addMenuItem(new PopupMenu.PopupSwitchMenuItem(_("Graphics Card '%s'").format('ATI')));
            }
        }
    },

    _updateHDDTemp: function() {
        let infoHDDTemp;
        if (this._commandHDDTemp) {
            infoHDDTemp = GLib.spawn_command_line_sync(this._commandHDDTemp);
            if (infoHDDTemp[0]) infoHDDTemp = this._getHDDTemp(infoHDDTemp[1].toString(), false);
        } else if (this._daemonHDDTemp) {
            infoHDDTemp = GLib.spawn_command_line_sync('nc localhost '+this._daemonHDDTemp);
            if (infoHDDTemp[0]) infoHDDTemp = this._getHDDTemp(infoHDDTemp[1].toString(), true);
        }
        if (infoHDDTemp) {
            for (let sensor in infoHDDTemp) {
                this._infoHDDTemp.push(infoHDDTemp[sensor]['temp']);                
                this._itemHDDTemp.addMenuItem(new PopupMenu.PopupSwitchMenuItem(_("Disk '%s'").format(infoHDDTemp[sensor]['label'])));
            }
        }
    },

    _updateHWMon: function() {
        let hwmonContents;
        let hwmonFile
        let hwmonName;
        for each (let hwmonDir in this._dirHWMon) {
            hwmonFile = hwmonDir+'name';
            if (GLib.file_test(hwmonFile, GLib.FileTest.EXISTS)) {
                hwmonContents = this._getContent(hwmonFile);
                hwmonName = hwmonContents[0] ? hwmonContents[1].toString().split('\n')[0] : null;
            }
            if (hwmonName) for (let i = 1; ; i++) {
                let infoHWMon = [];
                hwmonFile = hwmonDir+'temp'+i+'_label';
                if (GLib.file_test(hwmonFile, GLib.FileTest.EXISTS)) {
                    hwmonContents = this._getContent(hwmonFile);
                    if (hwmonContents[0]) infoHWMon['label'] = hwmonContents[1].toString().split('\n')[0];
                }
                hwmonFile = hwmonDir+'temp'+i+'_input';
                if (GLib.file_test(hwmonFile, GLib.FileTest.EXISTS)) {
                    hwmonContents = this._getContent(hwmonFile);
                    if (hwmonContents[0]) infoHWMon['current'] = parseInt(hwmonContents[1])/1000;
                }
                hwmonFile = hwmonDir+'temp'+i+'_max';
                if (GLib.file_test(hwmonFile, GLib.FileTest.EXISTS)) {
                    hwmonContents = this._getContent(hwmonFile);
                    if (hwmonContents[0]) infoHWMon['maximum'] = parseInt(hwmonContents[1])/1000;
                }
                hwmonFile = hwmonDir+'temp'+i+'_crit';
                if (GLib.file_test(hwmonFile, GLib.FileTest.EXISTS)) {
                    hwmonContents = this._getContent(hwmonFile);
                    if (hwmonContents[0]) infoHWMon['critical'] = parseInt(hwmonContents[1])/1000;
                }
                if (!infoHWMon['current'] && infoHWMon['critical']) infoHWMon['current'] = null;
                if (!infoHWMon['critical']) infoHWMon['critical'] = null;
                if (!infoHWMon['maximum']) infoHWMon['maximum'] = null;
                if (!infoHWMon['label']) infoHWMon['label'] = 'unknown';
                if (infoHWMon['current']) {
                    switch (hwmonName) {
                        case 'acpitz':
                            this._tempCPU = infoHWMon['current'];
                            this._infoHWMonACPITZ.push(infoHWMon['current']);
                            this._itemHWMonACPITZ.addMenuItem(new PopupMenu.PopupSwitchMenuItem(_("Current temperature")));
                            if (infoHWMon['critical']) {
                                this._infoHWMonACPITZ.push(infoHWMon['critical']);
                                this._itemHWMonACPITZ.addMenuItem(new PopupMenu.PopupSwitchMenuItem(_("Critical temperature")));
                            }
                            break;
                        case 'atk0110':
                            if (infoHWMon['label'] === 'CPU Temperature') this._tempCPU = infoHWMon['current'];
                            this._infoHWMonDevice.push({ current: infoHWMon['current'], maximum: infoHWMon['maximum'], critical: infoHWMon['critical'] });
                            this._itemHWMonDevice.addMenuItem(new PopupMenu.PopupSwitchMenuItem(_(infoHWMon['label'])));
                            break;
                        case 'sis5595':
                        case 'via686a':
                        case 'vt8231':
                        case 'k8temp':
                        case 'k10temp':
                        case 'fam15h_power':
                        case 'coretemp':
                        case 'i5k_amb':
                        case 'via-cputemp':
                            if (!this._tempCPUCores || this._tempCPUCores < infoHWMon['current']) this._tempCPUCores = infoHWMon['current'];
                            this._infoCPUCores.push({ current: infoHWMon['current'], maximum: infoHWMon['maximum'], critical: infoHWMon['critical'] });
                            this._itemCPUCores.addMenuItem(new PopupMenu.PopupSwitchMenuItem(_("CPU '%s'").format(infoHWMon['label'])));
                            break;
                            
                        case 'nouveau':
                            if (!this._tempGraphics || this._tempGraphics < infoHWMon['current']) this._tempGraphics = infoHWMon['current'];
                            this._infoGraphics.push({ current: infoHWMon['current'], maximum: infoHWMon['maximum'], critical: infoHWMon['critical'] });
                            this._itemGraphicCards.addMenuItem(new PopupMenu.PopupSwitchMenuItem(_("Graphics Card '%s'").format(hwmonName)));
                            break;
                        default:
                            this._infoHWMonDevice.push({ current: infoHWMon['current'], maximum: infoHWMon['maximum'], critical: infoHWMon['critical'] });
                            this._itemHWMonDevice.addMenuItem(new PopupMenu.PopupSwitchMenuItem(hwmonName+(" '%s'").format(infoHWMon['label'])));
                            break;
                    }
                } else {
                    if (i > 1) break;
                }
            }
        }
    },

    _updateNVIDIA: function() {
        let nvidiaOutput = GLib.spawn_command_line_sync(this._commandNVIDIA+' -q gpucoretemp -t')
        if (nvidiaOutput[0]) {
            let nvidiaTemp = nvidiaOutput[1].toString().split('\n')[0];
            if (nvidiaTemp > 0) {
                if (!this._tempGraphics || this._tempGraphics < nvidiaTemp) this._tempGraphics = nvidiaTemp;
                this._infoGraphics.push({ current: nvidiaTemp, maximum: null, critical: null });
                this._itemGraphicCards.addMenuItem(new PopupMenu.PopupSwitchMenuItem(_("Graphics Card '%s'").format('NVIDIA')));
            }
        }
    },

    _updateSensors: function () {
        this._infoCPUCores = [];
        this._infoGraphics = [];
        this._infoHDDTemp = [];
        this._infoHWMonACPITZ = [];
        this._infoHWMonDevice = [];
        this._tempCPU = null;
        this._tempCPUCores = null;
        this._tempGraphics = null;
        this._itemCPUCores.removeAll();
        this._itemGraphicCards.removeAll();
        this._itemHDDTemp.removeAll();
        this._itemHWMonACPITZ.removeAll();
        this._itemHWMonDevice.removeAll();
        if (this._dirHWMon) this._updateHWMon();
        if (this._commandHDDTemp || this._daemonHDDTemp) this._updateHDDTemp();
        if (this._commandATI) this._updateATI();
        if (this._commandNVIDIA) this._updateNVIDIA();
        this._onShowFahrenheit();
        this._itemNotAvailable.actor.visible = !this._itemCPUCores.actor.visible && !this._itemGraphicCards.actor.visible && !this._itemHDDTemp.actor.visible && !this._itemHWMonACPITZ.actor.visible && !this._itemHWMonDevice.actor.visible;
    }
});

function getExtension() {
    return PanelTools.addToStatusArea('dschaerfTemperature', new Indicator(), 0, 'right');
}
