/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2014  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;
const Mainloop = imports.mainloop;

const Clutter = imports.gi.Clutter;
const Soup = imports.gi.Soup;
const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;
const PanelTools = Extension.imports.panelTools;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    Weather
 */
// Soup session (see https://bugzilla.gnome.org/show_bug.cgi?id=661323#c64) (Simon Legner)
const _httpSession = new Soup.SessionAsync();
Soup.Session.prototype.add_feature.call(_httpSession, new Soup.ProxyResolverDefault());

const KEY_LOCATION = 'location';
const KEY_REFRESH_INTERVAL = 'refresh-interval';
const KEY_SHOW_SETTINGS = 'show-settings';
const KEY_SHOW_WEATHER_FORECAST = 'show-weather-forecast';
const KEY_WEATHER_FORECAST = 'weather-forecast';
const KEY_WEATHER_ICONS = 'weather-icons';
const KEY_WEATHER_LOCATIONS = 'weather-locations';
const KEY_WEATHER_UNITS = 'weather-units';
const KEY_WIND_DIRECTION = 'wind-direction';

//developer.yahoo.com/weather/#codetable
const WEATHER_CODE = [
    {   text: _("Tornado"),
        icon: 'weather-storm'
    },
    {   text: _("Tropical Storm"),
        icon: 'weather-storm'
    },
    {   text: _("Hurricane"),
        icon: 'weather-storm'
    },
    {   text: _("Severe Thunderstorms"),
        icon: 'weather-storm'
    },
    {   text: _("Thunderstorms"),
        icon: 'weather-storm'
    },
    {   text: _("Mixed Rain and Snow"),
        icon: 'weather-snow'
    },
    {   text: _("Mixed Rain and Sleet"),
        icon: 'weather-snow'
    },
    {   text: _("Mixed Snow and Sleet"),
        icon: 'weather-snow'
    },
    {   text: _("Freezing Drizzle"),
        icon: 'weather-showers'
    },
    {   text: _("Drizzle"),
        icon: 'weather-showers'
    },
    {   text: _("Freezing Rain"),
        icon: 'weather-showers'
    },
    {   text: _("Showers"),
        icon: 'weather-showers'
    },
    {   text: _("Showers"),
        icon: 'weather-showers'
    },
    {   text: _("Snow Flurries"),
        icon: 'weather-snow'
    },
    {   text: _("Light Snow Showers"),
        icon: 'weather-snow'
    },
    {   text: _("Blowing Snow"),
        icon: 'weather-snow'
    },
    {   text: _("Snow"),
        icon: 'weather-snow'
    },
    {   text: _("Hail"),
        icon: 'weather-snow'
    },
    {   text: _("Sleet"),
        icon: 'weather-snow'
    },
    {   text: _("Dust"),
        icon: 'weather-fog'
    },
    {   text: _("Foggy"),
        icon: 'weather-fog'
    },
    {   text: _("Haze"),
        icon: 'weather-fog'
    },
    {   text: _("Smoky"),
        icon: 'weather-fog'
    },
    {   text: _("Blustery"),
        icon: 'weather-few-clouds'
    },
    {   text: _("Windy"),
        icon: 'weather-few-clouds'
    },
    {   text: _("Cold"),
        icon: 'weather-few-clouds'
    },
    {   text: _("Cloudy"),
        icon: 'weather-overcast'
    },
    {   text: _("Mostly Cloudy"),
        icon: 'weather-few-clouds-night'
    },
    {   text: _("Mostly Cloudy"),
        icon: 'weather-few-clouds'
    },
    {   text: _("Partly Cloudy"),
        icon: 'weather-few-clouds-night'
    },
    {   text: _("Partly Cloudy"),
        icon: 'weather-few-clouds'
    },
    {   text: _("Clear"),
        icon: 'weather-clear-night'
    },
    {   text: _("Sunny"),
        icon: 'weather-clear'
    },
    {   text: _("Fair"),
        icon: 'weather-clear-night'
    },
    {   text: _("Fair"),
        icon: 'weather-clear'
    },
    {   text: _("Mixed Rain and Hail"),
        icon: 'weather-showers'
    },
    {   text: _("Hot"),
        icon: 'weather-clear'
    },
    {   text: _("Isolated Thunderstorms"),
        icon: 'weather-storm'
    },
    {   text: _("Scattered Thunderstorms"),
        icon: 'weather-storm'
    },
    {   text: _("Scattered Thunderstorms"),
        icon: 'weather-showers-scattered'
    },
    {   text: _("Scattered Showers"),
        icon: 'weather-showers-scattered'
    },
    {   text: _("Heavy Snow"),
        icon: 'weather-snow'
    },
    {   text: _("Scattered Snow Showers"),
        icon: 'weather-snow'
    },
    {   text: _("Heavy Snow"),
        icon: 'weather-snow'
    },
    {   text: _("Partly Cloudy"),
        icon: 'weather-few-clouds'
    },
    {   text: _("Thundershowers"),
        icon: 'weather-storm'
    },
    {   text: _("Snow Showers"),
        icon: 'weather-snow'
    },
    {   text: _("Isolated Thundershowers"),
        icon: 'weather-storm'
    },
    {   text: _("Not available"),
        icon: 'weather-severe-alert'
    }
];

const WEATHER_COMPASS_ARROWS = [
    '\u2193',
    '\u2199',
    '\u2190',
    '\u2196',
    '\u2191',
    '\u2197',
    '\u2192',
    '\u2198',
    '\u2193'
];

const WEATHER_COMPASS_LETTERS = [
    'N',
    'NNE',
    'NE',
    'ENE',
    'E',
    'ESE',
    'SE',
    'SSE',
    'S',
    'SSW',
    'SW',
    'WSW',
    'W',
    'WNW',
    'NW',
    'NNW',
    'N'
];

const WEATHER_DAY = [
    _("Sunday"),
    _("Monday"),
    _("Tuesday"),
    _("Wednesday"),
    _("Thursday"),
    _("Friday"),
    _("Saturday")
];

const WEATHER_PRESSURE = [
    '',
    '\u2934',
    '\u2935'
];

const Indicator = new Lang.Class({
    Name: 'WeatherIndicator',
    Extends: PanelTools.Button,

    _init: function() {
        this.parent(0.25, 'WeatherIndicator', { icon_name: 'view-refresh-symbolic', text: '...' });
        this.event = undefined;
        this._interface = Convenience.getSettings('org.gnome.desktop.interface');
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._createSubMenu();
        this._onShowSettings();
        this._onShowWeatherForecast();
        this._onWeatherLocations();
        this._onRefreshInterval();
        this._connectInterface = this._interface.connect('changed::clock-format', Lang.bind(this, this._onClockFormat));
        this._connectSettings = [
            this._settings.connect('changed::'+KEY_LOCATION, Lang.bind(this, this._onLocation)),
            this._settings.connect('changed::'+KEY_REFRESH_INTERVAL, Lang.bind(this, this._onRefreshInterval)),
            this._settings.connect('changed::'+KEY_SHOW_SETTINGS, Lang.bind(this, this._onShowSettings)),
            this._settings.connect('changed::'+KEY_SHOW_WEATHER_FORECAST, Lang.bind(this, this._onShowWeatherForecast)),
            this._settings.connect('changed::'+KEY_WEATHER_FORECAST, Lang.bind(this, this._onWeatherForecast)),
            this._settings.connect('changed::'+KEY_WEATHER_ICONS, Lang.bind(this, this._onWeatherIcons)),
            this._settings.connect('changed::'+KEY_WEATHER_LOCATIONS, Lang.bind(this, this._onWeatherLocations)),
            this._settings.connect('changed::'+KEY_WEATHER_UNITS, Lang.bind(this, this._onWeatherUnits)),
            this._settings.connect('changed::'+KEY_WIND_DIRECTION, Lang.bind(this, this._onWindDirection))
        ];
        this.actor.connect('destroy', Lang.bind(this, function() {
            if (this.event) Mainloop.source_remove(this.event);
            for each(let connection in this._connectSettings) {
                this._settings.disconnect(connection);
            }
            this._interface.disconnect(this._connectInterface);
        }));
    },

    _createCurrentWeather: function() {
        this._weatherCurrentBuildButton = new St.Button({ style_class: 'weathermenu-current-time-button' });
        this._weatherCurrentBuildIcon = new St.Icon({ icon_name: 'view-refresh' });
        this._weatherCurrentBuildButton.add_actor(this._weatherCurrentBuildIcon);
        this._weatherCurrentBuildButton.connect('clicked', Lang.bind(this, this._updateWeather));
        this._weatherCurrentBuildText = new St.Label({ text: '...', style_class: 'weathermenu-current-time-label' });
        this._weatherCurrentFeels = new St.Label({ text: '...' });
        this._weatherCurrentHumidity = new St.Label({ text: '...' });
        this._weatherCurrentIcon = new St.Bin();
        this._weatherCurrentIcon.child = this._getIcon();
        this._weatherCurrentPressure = new St.Label({ text: '...' });
        this._weatherCurrentSummary = new St.Label({ text: _("Loading..."), style_class: 'weathermenu-current-summary-label' });
        this._weatherCurrentSunriseIcon = new St.Icon({ icon_name: 'weather-clear', style_class: 'weathermenu-current-time-icon' });
        this._weatherCurrentSunriseText = new St.Label({ text: '...', style_class: 'weathermenu-current-time-label' });
        this._weatherCurrentSunsetIcon = new St.Icon({ icon_name: 'weather-clear-night', style_class: 'weathermenu-current-time-icon' });
        this._weatherCurrentSunsetText = new St.Label({ text: '...', style_class: 'weathermenu-current-time-label' });
        this._weatherCurrentTemperatureText = new St.Label({ text: _("Temperature:"), style_class: 'weathermenu-current-average-caption' });
        this._weatherCurrentTemperatureValue = new St.Label({ text: '...' });
        this._weatherCurrentVisibility = new St.Label({ text: '...' });
        this._weatherCurrentWind = new St.Label({ text: '...' });
        if (St.IconType) {
            this._weatherCurrentBuildIcon.icon_type = St.IconType.FULLCOLOR;
            this._weatherCurrentSunriseIcon.icon_type = St.IconType.FULLCOLOR;
            this._weatherCurrentSunsetIcon.icon_type = St.IconType.FULLCOLOR;
        }
        let boxCurrent = new St.BoxLayout({ vertical: true, name: 'weatherCurrentArea' });
        this._binCurrentWeather.child = boxCurrent;
        let boxTime = new St.BoxLayout({ vertical: false });
        boxCurrent.add(boxTime, { expand: true });
        boxCurrent.add(this._weatherCurrentSummary);
        boxTime.add(this._weatherCurrentSunriseIcon);
        boxTime.add(this._weatherCurrentSunriseText);
        boxTime.add(this._weatherCurrentSunsetIcon);
        boxTime.add(this._weatherCurrentSunsetText);
        boxTime.add(this._weatherCurrentBuildButton);
        boxTime.add(this._weatherCurrentBuildText);
        let boxAverage = new St.BoxLayout({ vertical: false });
        boxCurrent.add(boxAverage);
        boxAverage.add(this._weatherCurrentIcon);
        let boxAverageCaption = new St.BoxLayout({ vertical: true, style_class: 'weathermenu-current-average' });
        boxAverage.add(boxAverageCaption, { expand: true });
        boxAverageCaption.add(this._weatherCurrentTemperatureText);
        boxAverageCaption.add(new St.Label({ text: _("Feels like:"), style_class: 'weathermenu-current-average-caption' }));
        boxAverageCaption.add(new St.Label({ text: _("Humidity:"), style_class: 'weathermenu-current-average-caption' }));
        boxAverageCaption.add(new St.Label({ text: _("Visibility:"), style_class: 'weathermenu-current-average-caption' }));
        boxAverageCaption.add(new St.Label({ text: _("Wind:"), style_class: 'weathermenu-current-average-caption' }));
        boxAverageCaption.add(new St.Label({ text: _("Pressure:"), style_class: 'weathermenu-current-average-caption' }));
        let boxAverageValue = new St.BoxLayout({ vertical: true, style_class: 'weathermenu-current-average' });
        boxAverage.add(boxAverageValue);
        boxAverageValue.add(this._weatherCurrentTemperatureValue);
        boxAverageValue.add(this._weatherCurrentFeels);
        boxAverageValue.add(this._weatherCurrentHumidity);
        boxAverageValue.add(this._weatherCurrentVisibility);
        boxAverageValue.add(this._weatherCurrentWind);
        boxAverageValue.add(this._weatherCurrentPressure);
    },

    _createForecastWeather: function() {
        let boxForecast = new St.BoxLayout({ vertical: true, name: 'weatherForecastArea' });
        this._binForecastWeather.child = boxForecast;
        this._weatherForecast = [];
        for (let i = 0; i <= this._settings.get_enum(KEY_WEATHER_FORECAST) + 1; i++) {
            let box = new St.BoxLayout({ vertical: true });
            boxForecast.add(box);
            let forecast = {};
            forecast.Day = new St.Label({ style_class: 'weathermenu-forecast-header' });
            forecast.Icon = new St.Icon({ icon_name: 'view-refresh-symbolic', style_class: 'weathermenu-forecast-icon' });
            forecast.Summary = new St.Label({ style_class: 'weathermenu-forecast-summary' });
            forecast.Temperature = new St.Label({ style_class: 'weathermenu-forecast-temperature' });
            this._weatherForecast[i] = forecast;
            box.add(forecast.Day);
            let boxSummary = new St.BoxLayout({ vertical: false });
            box.add(boxSummary);
            boxSummary.add(forecast.Icon);
            boxSummary.add(forecast.Temperature);
            boxSummary.add(forecast.Summary);
            boxSummary.add(forecast.Day);
        }
    },

    _createSubMenu: function() {
        let hbox = new St.BoxLayout({ vertical: false });
        this.menu.box.add_child(hbox);
        let vbox = new St.BoxLayout({ vertical: true });
        hbox.add(vbox);
        this._itemLocation = new PanelTools.PopupSubMenuMenuItem('...', false, { span: -1 });
        this._itemLocation.label.style_class = 'weathermenu-current-location-label';
        this.menu.addMenuItem(this._itemLocation);
        this._itemLocation.actor.reparent(vbox);
        this._binCurrentWeather = new St.Bin({ x_align: St.Align.START, x_fill: true, y_align: St.Align.START, y_fill: true });
        vbox.add(this._binCurrentWeather);
        this._boxSettings = new St.BoxLayout({ vertical: true });
        vbox.add(this._boxSettings);
        let item = new PopupMenu.PopupSeparatorMenuItem();
        item.actor.reparent(this._boxSettings);
        item = this.menu.addAction(_("Settings for Weather"), Lang.bind(this, function() {
            Main.overview.hide();
            Main.Util.spawnCommandLine('gnome-shell-extension-prefs '+Extension['uuid']);
        }));
        item.actor.reparent(this._boxSettings);
        this._boxForecast = new St.BoxLayout({ vertical: false });
        hbox.add(this._boxForecast);
        item = new PanelTools.VerticalSeparator();
        this._boxForecast.add(item.actor);
        this._binForecastWeather = new St.Bin({ x_align: St.Align.START, x_fill: true, y_align: St.Align.START, y_fill: true });
        this._boxForecast.add(this._binForecastWeather);
        this._createCurrentWeather();
        this._createForecastWeather();
    },

    _getIcon: function(weather) {
        if (weather) {
            if (this._settings.get_string(KEY_WEATHER_ICONS) === 'default') {
                let currentDate = new Date();
                let sunriseDate = new Date(currentDate.toDateString() +' '+ weather.astronomy.sunrise);
                let sunsetDate = new Date(currentDate.toDateString() +' '+ weather.astronomy.sunset);
                return St.TextureCache.get_default().load_uri_async('http://l.yimg.com/a/i/us/nws/weather/gr/'+weather.item.condition.code+((currentDate > sunriseDate && currentDate < sunsetDate) ? 'd' : 'n')+'.png', 175, 126, true);
            } else {
                let icon = new St.Icon({ icon_name: WEATHER_CODE[weather.item.condition.code]['icon'], style_class: 'weathermenu-current-icon' });
                if (St.IconType) icon.icon_type = St.IconType.FULLCOLOR;
                return icon;
            }
        } else {
            let icon = new St.Icon({ icon_name: 'view-refresh', style_class: 'weathermenu-current-icon' });
            if (St.IconType) icon.icon_type = St.IconType.FULLCOLOR;
            return icon;
        }
    },

    _getLocation: function(param) {
        let locations = this._settings.get_strv(KEY_WEATHER_LOCATIONS);
        let i = this._settings.get_int(KEY_LOCATION);
        if (i >= locations.length) return (param = 0 ? '' : _("Invalid city"));
        return locations[i].split('>')[param];
    },

    _getTime: function(time) {
        let hours = parseInt(time.split(':')[0]);
        let minutes = parseInt(time.split(':')[1]);
        if (this._interface.get_string('clock-format') === '24h') {
            if (time.indexOf(' pm') > 0 && hours < 12) hours += 12;
            if (time.indexOf(' am') > 0 && hours === 12) hours = 0;
            return hours+':'+(minutes < 10 ? '0' : '')+minutes;
        } else {
            if (time.indexOf(' pm') > 0 || time.indexOf(' am') > 0) return time;
            if (hours >= 13) return (hours -= 12)+':'+(minutes < 10 ? '0' : '')+minutes+' pm';
            return hours+':'+(minutes < 10 ? '0' : '')+minutes+' am';
        }
    },

    _getTimeBuild: function(time) {
        let buildTime = time.split(' ');
        let updateTime = new Date();
        return this._getTime(buildTime[4]+' '+buildTime[5])+' ('+this._getTime(updateTime.getHours()+':'+updateTime.getMinutes())+')';
    },

    _getWindSpeed: function(weather) {
        if (!weather) return 'N/A';
        let windDirection;
        if (this._settings.get_string(KEY_WIND_DIRECTION) === 'letters') {
            windDirection = WEATHER_COMPASS_LETTERS[Math.round(weather.wind.direction / 22.5)];
        } else {
            windDirection = WEATHER_COMPASS_ARROWS[Math.round(weather.wind.direction / 45)];
        }
        if (!weather.wind.speed) return '\u2013';
        if (weather.wind.speed === 0 || !weather.wind.direction) return weather.wind.speed+' '+weather.units.speed;
        return windDirection+' '+weather.wind.speed+' '+weather.units.speed;
    },

    _loadWeather: function(fun) {
        let unit = (this._settings.get_string(KEY_WEATHER_UNITS) === 'metric' ? 'c' : 'f');
        let message = Soup.Message.new('GET', encodeURI('https://query.yahooapis.com/v1/public/yql?format=json&q=select*from weather.forecast where woeid='+this._getLocation(0)+' and u="'+unit+'"'));
        _httpSession.queue_message(message, Lang.bind(this, function(_httpSession, message) {
            if (!message.response_body.data) fun.call(this, false);
            try {
                fun.call(this, JSON.parse(message.response_body.data));
            } catch(e) {
                fun.call(this, false);
            }
        }));
    },

    _onClockFormat: function() {
        this._onRefreshInterval();
    },

    _onLocation: function() {
        this._updateLocation();
        this._updateWeather();
    },

    _onRefreshInterval: function() {
        if (this.event) Mainloop.source_remove(this.event);
        this.event = Mainloop.timeout_add_seconds(this._settings.get_string(KEY_REFRESH_INTERVAL), Lang.bind(this, function() {
            this._updateWeather();
            return true;
        }));
        this._updateWeather();
    },

    _onShowSettings: function() {
        this._boxSettings.visible = this._settings.get_boolean(KEY_SHOW_SETTINGS);
    },

    _onShowWeatherForecast: function() {
        this._boxForecast.visible = this._settings.get_boolean(KEY_SHOW_WEATHER_FORECAST);
    },

    _onWeatherForecast: function() {
        this._createForecastWeather();
        this._updateWeather();
    },

    _onWeatherIcons: function() {
        this._weatherCurrentIcon.child = this._getIcon(this._weather);
    },

    _onWeatherLocations: function() {
        this._itemLocation.menu.removeAll();
        let locations = this._settings.get_strv(KEY_WEATHER_LOCATIONS);
        for (let i = 0; i < locations.length; i++) {
            let item = new PanelTools.PopupMenuItem(locations[i].split('>')[1]);
            this._itemLocation.menu.addMenuItem(item);
            item.location = i;
            item.connect('activate', Lang.bind(this, function() {
                this._itemLocation.selected.setSelected(false);
                this._settings.set_int(KEY_LOCATION, item.location);
                this._itemLocation.label.text = this._settings.get_strv(KEY_WEATHER_LOCATIONS)[item.location].split('>')[1];
                this._itemLocation.selected = item;
                this._itemLocation.selected.setSelected(true);
                this.menu.open(1);
            }));
        }
        this._updateLocation();
    },

    _onWindDirection: function() {
        this._weatherCurrentWind.set_text(this._getWindSpeed(this._weather));
    },

    _onWeatherUnits: function() {
        this._updateWeather();
    },

    _updateLocation: function() {
        let menuItems = this._itemLocation.menu._getMenuItems();
        if (this._itemLocation.selected)this._itemLocation.selected.setSelected(false);
        for each(let item in menuItems) {
            if (item.location === this._settings.get_int(KEY_LOCATION)) {
                this._itemLocation.label.text = item.label.text;
                this._itemLocation.selected = item;
                this._itemLocation.selected.setSelected(true);
            }
        }
    },

    _updateWeather: function() {
        this._loadWeather(function(json) {
            if (!json) return;
            let weather = json.query.results.channel;
            if (typeof weather[0] !== 'undefined') weather = weather[0];
            if (weather.item.condition.code > 48) weather.item.condition.code = 49;
            this.icon.icon_name = WEATHER_CODE[weather.item.condition.code]['icon']+'-symbolic';
            this.label.text = weather.item.condition.temp+'\u00B0'+weather.units.temperature;
            this._weatherCurrentBuildText.set_text(this._getTimeBuild(weather.lastBuildDate));
            this._weatherCurrentFeels.set_text(weather.wind.chill+'\u00B0'+weather.units.temperature);
            this._weatherCurrentHumidity.set_text(weather.atmosphere.humidity+'%');
            this._weatherCurrentIcon.child = this._getIcon(weather);
            this._weatherCurrentPressure.set_text(weather.atmosphere.pressure+' '+weather.units.pressure+((weather.atmosphere.rising) ? ' '+WEATHER_PRESSURE[weather.atmosphere.rising] : ''));
            this._weatherCurrentSummary.set_text(_(WEATHER_CODE[weather.item.condition.code]['text'])+', '+weather.item.condition.temp+'\u00B0'+weather.units.temperature);
            this._weatherCurrentSunriseText.set_text(this._getTime(weather.astronomy.sunrise));
            this._weatherCurrentSunsetText.set_text(this._getTime(weather.astronomy.sunset));
            this._weatherCurrentTemperatureValue.set_text('\u2191 '+weather.item.forecast[0].high+'\u00B0'+weather.units.temperature+'   \u2193 '+weather.item.forecast[0].low+'\u00B0'+weather.units.temperature);
            this._weatherCurrentVisibility.set_text((weather.atmosphere.visibility > 0 ? weather.atmosphere.visibility+' '+weather.units.distance : '?' ));
            this._weatherCurrentWind.set_text(this._getWindSpeed(weather));
            for (let i = 0; i < this._weatherForecast.length; i++) {
                let actualDate = new Date();
                let forecastDate = new Date(weather.item.forecast[i].date);
                let dayLeft = Math.floor((actualDate.getTime() - forecastDate.getTime()) / 1000 / 60 / 60 / 24);
                let date_string = _("Today");
                if (dayLeft === -1) {
                    date_string = _("Tomorrow");
                } else if (dayLeft < -1) {
                    date_string = _("In %s Days").format(-1 * dayLeft);
                } else if (dayLeft === 1) {
                    date_string = _("Yesterday");
                } else if (dayLeft > 1) {
                    date_string = _("%s Days ago").format(dayLeft);
                }
                this._weatherForecast[i].Day.text = date_string+' ('+_(WEATHER_DAY[new Date(weather.item.forecast[i].date).getDay()])+')';
                this._weatherForecast[i].Icon.icon_name = WEATHER_CODE[weather.item.forecast[i].code]['icon']+'-symbolic';
                this._weatherForecast[i].Temperature.text = '\u2191 '+weather.item.forecast[i].high+'\u00B0'+weather.units.temperature+'   \u2193 '+weather.item.forecast[i].low+'\u00B0'+weather.units.temperature;
                this._weatherForecast[i].Summary.text = _(WEATHER_CODE[weather.item.forecast[i].code]['text']);
            }
            this._weather = weather;
        });
    }
});

function getExtension() {
    return PanelTools.addToStatusArea('dschaerfWeather', new Indicator(), 1, 'center');
}
